package com.qmth.exam.reserve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.exam.reserve.bean.student.StudentInfo;
import com.qmth.exam.reserve.bean.student.WechatBindReq;
import com.qmth.exam.reserve.dao.StudentDao;
import com.qmth.exam.reserve.entity.StudentEntity;
import com.qmth.exam.reserve.service.StudentService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

@Service
public class StudentServiceImpl extends ServiceImpl<StudentDao, StudentEntity> implements StudentService {

    private static final Logger log = LoggerFactory.getLogger(StudentServiceImpl.class);

    @Override
    public StudentEntity findByStudentCode(Long orgId, String studentCode) {
        if (StringUtils.isEmpty(studentCode)) {
            throw new StatusException("学号不能为空");
        }

        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(StudentEntity::getStudentCode, studentCode);
        if (orgId != null) {
            wrapper.lambda().eq(StudentEntity::getOrgId, orgId);
        }

        return baseMapper.selectOne(wrapper);
    }

    @Override
    public StudentEntity findByOpenIdAndUid(String openId, String uid) {
        if (StringUtils.isEmpty(openId)) {
            throw new StatusException("微信OID不能为空");
        }

        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(StudentEntity::getOpenId, openId);
        if (StringUtils.isNotEmpty(uid)) {
            wrapper.lambda().eq(StudentEntity::getUid, uid);
        }

        return baseMapper.selectOne(wrapper);
    }

    @Override
    public StudentInfo findInfoByStudentId(Long studentId) {
        StudentEntity entity = baseMapper.selectById(studentId);
        if (entity == null) {
            throw new StatusException("学生信息不存在");
        }

        StudentInfo info = new StudentInfo();
        info.setOrgId(entity.getOrgId());
        info.setId(entity.getId());
        info.setName(entity.getName());
        info.setStudentCode(entity.getStudentCode());
        info.setIdentityNumber(entity.getIdentityNumber());
        info.setPhotoPath(entity.getPhotoPath());
        info.setGender(entity.getGender());
        info.setApplyNumber(entity.getApplyNumber());
        info.setApplyTaskId(entity.getApplyTaskId());
        info.setCategoryId(entity.getCategoryId());
        info.setOpenId(entity.getOpenId());
        info.setUid(entity.getUid());
        return info;
    }

    @Override
    public void bindingWechat(WechatBindReq req) {
        StudentEntity stu = baseMapper.selectById(req.getStudentId());
        if (stu == null) {
            throw new StatusException("学生信息不存在");
        }

        if (StringUtils.isNotEmpty(stu.getOpenId())) {
            throw new StatusException("微信账号已绑定");
        }

        stu.setOpenId(req.getOpenId());
        stu.setUid(req.getUid());
        baseMapper.updateById(stu);
        log.info("[WECHAT_BINDING] studentId:{} openId:{} uid:{}", stu.getId(), stu.getOpenId(), stu.getUid());
    }

    @Override
    public void unbindWechatByStudentId(Long studentId) {
        StudentEntity stu = baseMapper.selectById(studentId);
        if (stu == null) {
            throw new StatusException("学生信息不存在");
        }

        log.info("[WECHAT_UNBIND] studentId:{} openId:{} uid:{}", stu.getId(), stu.getOpenId(), stu.getUid());
        if (StringUtils.isEmpty(stu.getOpenId())) {
            return;
        }

        stu.setOpenId(null);
        stu.setUid(null);
        baseMapper.updateById(stu);
    }

}
